import { FunctionComponent, useState } from "react";
import Link from "next/link";
import { useRouter } from "next/router";
import { ErrorMessage, Formik } from "formik";
import { useMutation } from "react-query";

import { fetch, FetchError, FetchResponse } from "../../utils/dataAccess";
import { NombreVentes } from "../../types/NombreVentes";

interface Props {
  nombreventes?: NombreVentes;
}

interface SaveParams {
  values: NombreVentes;
}

interface DeleteParams {
  id: string;
}

const saveNombreVentes = async ({ values }: SaveParams) =>
  await fetch<NombreVentes>(!values["@id"] ? "/ventes/nombre" : values["@id"], {
    method: !values["@id"] ? "POST" : "PUT",
    body: JSON.stringify(values),
  });

const deleteNombreVentes = async (id: string) =>
  await fetch<NombreVentes>(id, { method: "DELETE" });

export const Form: FunctionComponent<Props> = ({ nombreventes }) => {
  const [, setError] = useState<string | null>(null);
  const router = useRouter();

  const saveMutation = useMutation<
    FetchResponse<NombreVentes> | undefined,
    Error | FetchError,
    SaveParams
  >((saveParams) => saveNombreVentes(saveParams));

  const deleteMutation = useMutation<
    FetchResponse<NombreVentes> | undefined,
    Error | FetchError,
    DeleteParams
  >(({ id }) => deleteNombreVentes(id), {
    onSuccess: () => {
      router.push("/nombreventess");
    },
    onError: (error) => {
      setError(`Error when deleting the resource: ${error}`);
      console.error(error);
    },
  });

  const handleDelete = () => {
    if (!nombreventes || !nombreventes["@id"]) return;
    if (!window.confirm("Are you sure you want to delete this item?")) return;
    deleteMutation.mutate({ id: nombreventes["@id"] });
  };

  return (
    <div className="container mx-auto px-4 max-w-2xl mt-4">
      <Link
        href="/nombreventess"
        className="text-sm text-cyan-500 font-bold hover:text-cyan-700"
      >
        {`< Back to list`}
      </Link>
      <h1 className="text-3xl my-2">
        {nombreventes
          ? `Edit NombreVentes ${nombreventes["@id"]}`
          : `Create NombreVentes`}
      </h1>
      <Formik
        initialValues={
          nombreventes
            ? {
                ...nombreventes,
              }
            : new NombreVentes()
        }
        validate={() => {
          const errors = {};
          // add your validation logic here
          return errors;
        }}
        onSubmit={(values, { setSubmitting, setStatus, setErrors }) => {
          const isCreation = !values["@id"];
          saveMutation.mutate(
            { values },
            {
              onSuccess: () => {
                setStatus({
                  isValid: true,
                  msg: `Element ${isCreation ? "created" : "updated"}.`,
                });
                router.push("/ventes/nombre");
              },
              onError: (error) => {
                setStatus({
                  isValid: false,
                  msg: `${error.message}`,
                });
                if ("fields" in error) {
                  setErrors(error.fields);
                }
              },
              onSettled: () => {
                setSubmitting(false);
              },
            }
          );
        }}
      >
        {({
          values,
          status,
          errors,
          touched,
          handleChange,
          handleBlur,
          handleSubmit,
          isSubmitting,
        }) => (
          <form className="shadow-md p-4" onSubmit={handleSubmit}>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="nombreventes_date"
              >
                date
              </label>
              <input
                name="date"
                id="nombreventes_date"
                value={values.date?.toLocaleString() ?? ""}
                type="dateTime"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.date && touched.date ? "border-red-500" : ""
                }`}
                aria-invalid={errors.date && touched.date ? "true" : undefined}
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="date"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="nombreventes_quantite"
              >
                quantite
              </label>
              <input
                name="quantite"
                id="nombreventes_quantite"
                value={values.quantite ?? ""}
                type="number"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.quantite && touched.quantite ? "border-red-500" : ""
                }`}
                aria-invalid={
                  errors.quantite && touched.quantite ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="quantite"
              />
            </div>
            {status && status.msg && (
              <div
                className={`border px-4 py-3 my-4 rounded ${
                  status.isValid
                    ? "text-cyan-700 border-cyan-500 bg-cyan-200/50"
                    : "text-red-700 border-red-400 bg-red-100"
                }`}
                role="alert"
              >
                {status.msg}
              </div>
            )}
            <button
              type="submit"
              className="inline-block mt-2 bg-cyan-500 hover:bg-cyan-700 text-sm text-white font-bold py-2 px-4 rounded"
              disabled={isSubmitting}
            >
              Submit
            </button>
          </form>
        )}
      </Formik>
      <div className="flex space-x-2 mt-4 justify-end">
        {nombreventes && (
          <button
            className="inline-block mt-2 border-2 border-red-400 hover:border-red-700 hover:text-red-700 text-sm text-red-400 font-bold py-2 px-4 rounded"
            onClick={handleDelete}
          >
            Delete
          </button>
        )}
      </div>
    </div>
  );
};
