import { FunctionComponent, useState } from "react";
import Link from "next/link";
import { useRouter } from "next/router";
import { ErrorMessage, Formik } from "formik";
import { useMutation } from "react-query";

import { fetch, FetchError, FetchResponse } from "../../utils/dataAccess";
import { RegionVentes } from "../../types/RegionVentes";

interface Props {
  regionventes?: RegionVentes;
}

interface SaveParams {
  values: RegionVentes;
}

interface DeleteParams {
  id: string;
}

const saveRegionVentes = async ({ values }: SaveParams) =>
  await fetch<RegionVentes>(
    !values["@id"] ? "/ventes/regions" : values["@id"],
    {
      method: !values["@id"] ? "POST" : "PUT",
      body: JSON.stringify(values),
    }
  );

const deleteRegionVentes = async (id: string) =>
  await fetch<RegionVentes>(id, { method: "DELETE" });

export const Form: FunctionComponent<Props> = ({ regionventes }) => {
  const [, setError] = useState<string | null>(null);
  const router = useRouter();

  const saveMutation = useMutation<
    FetchResponse<RegionVentes> | undefined,
    Error | FetchError,
    SaveParams
  >((saveParams) => saveRegionVentes(saveParams));

  const deleteMutation = useMutation<
    FetchResponse<RegionVentes> | undefined,
    Error | FetchError,
    DeleteParams
  >(({ id }) => deleteRegionVentes(id), {
    onSuccess: () => {
      router.push("/regionventess");
    },
    onError: (error) => {
      setError(`Error when deleting the resource: ${error}`);
      console.error(error);
    },
  });

  const handleDelete = () => {
    if (!regionventes || !regionventes["@id"]) return;
    if (!window.confirm("Are you sure you want to delete this item?")) return;
    deleteMutation.mutate({ id: regionventes["@id"] });
  };

  return (
    <div className="container mx-auto px-4 max-w-2xl mt-4">
      <Link
        href="/regionventess"
        className="text-sm text-cyan-500 font-bold hover:text-cyan-700"
      >
        {`< Back to list`}
      </Link>
      <h1 className="text-3xl my-2">
        {regionventes
          ? `Edit RegionVentes ${regionventes["@id"]}`
          : `Create RegionVentes`}
      </h1>
      <Formik
        initialValues={
          regionventes
            ? {
                ...regionventes,
              }
            : new RegionVentes()
        }
        validate={() => {
          const errors = {};
          // add your validation logic here
          return errors;
        }}
        onSubmit={(values, { setSubmitting, setStatus, setErrors }) => {
          const isCreation = !values["@id"];
          saveMutation.mutate(
            { values },
            {
              onSuccess: () => {
                setStatus({
                  isValid: true,
                  msg: `Element ${isCreation ? "created" : "updated"}.`,
                });
                router.push("/ventes/regions");
              },
              onError: (error) => {
                setStatus({
                  isValid: false,
                  msg: `${error.message}`,
                });
                if ("fields" in error) {
                  setErrors(error.fields);
                }
              },
              onSettled: () => {
                setSubmitting(false);
              },
            }
          );
        }}
      >
        {({
          values,
          status,
          errors,
          touched,
          handleChange,
          handleBlur,
          handleSubmit,
          isSubmitting,
        }) => (
          <form className="shadow-md p-4" onSubmit={handleSubmit}>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="regionventes_region"
              >
                region
              </label>
              <input
                name="region"
                id="regionventes_region"
                value={values.region ?? ""}
                type="number"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.region && touched.region ? "border-red-500" : ""
                }`}
                aria-invalid={
                  errors.region && touched.region ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="region"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="regionventes_count"
              >
                count
              </label>
              <input
                name="count"
                id="regionventes_count"
                value={values.count ?? ""}
                type="number"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.count && touched.count ? "border-red-500" : ""
                }`}
                aria-invalid={
                  errors.count && touched.count ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="count"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="regionventes_regionName"
              >
                regionName
              </label>
              <input
                name="regionName"
                id="regionventes_regionName"
                value={values.regionName ?? ""}
                type="text"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.regionName && touched.regionName
                    ? "border-red-500"
                    : ""
                }`}
                aria-invalid={
                  errors.regionName && touched.regionName ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="regionName"
              />
            </div>
            {status && status.msg && (
              <div
                className={`border px-4 py-3 my-4 rounded ${
                  status.isValid
                    ? "text-cyan-700 border-cyan-500 bg-cyan-200/50"
                    : "text-red-700 border-red-400 bg-red-100"
                }`}
                role="alert"
              >
                {status.msg}
              </div>
            )}
            <button
              type="submit"
              className="inline-block mt-2 bg-cyan-500 hover:bg-cyan-700 text-sm text-white font-bold py-2 px-4 rounded"
              disabled={isSubmitting}
            >
              Submit
            </button>
          </form>
        )}
      </Formik>
      <div className="flex space-x-2 mt-4 justify-end">
        {regionventes && (
          <button
            className="inline-block mt-2 border-2 border-red-400 hover:border-red-700 hover:text-red-700 text-sm text-red-400 font-bold py-2 px-4 rounded"
            onClick={handleDelete}
          >
            Delete
          </button>
        )}
      </div>
    </div>
  );
};
