import { FunctionComponent, useState } from "react";
import Link from "next/link";
import { useRouter } from "next/router";
import { ErrorMessage, Formik } from "formik";
import { useMutation } from "react-query";

import { fetch, FetchError, FetchResponse } from "../../utils/dataAccess";
import { Vente } from "../../types/Vente";

interface Props {
  vente?: Vente;
}

interface SaveParams {
  values: Vente;
}

interface DeleteParams {
  id: string;
}

const saveVente = async ({ values }: SaveParams) =>
  await fetch<Vente>(!values["@id"] ? "/ventes" : values["@id"], {
    method: !values["@id"] ? "POST" : "PUT",
    body: JSON.stringify(values),
  });

const deleteVente = async (id: string) =>
  await fetch<Vente>(id, { method: "DELETE" });

export const Form: FunctionComponent<Props> = ({ vente }) => {
  const [, setError] = useState<string | null>(null);
  const router = useRouter();

  const saveMutation = useMutation<
    FetchResponse<Vente> | undefined,
    Error | FetchError,
    SaveParams
  >((saveParams) => saveVente(saveParams));

  const deleteMutation = useMutation<
    FetchResponse<Vente> | undefined,
    Error | FetchError,
    DeleteParams
  >(({ id }) => deleteVente(id), {
    onSuccess: () => {
      router.push("/ventes");
    },
    onError: (error) => {
      setError(`Error when deleting the resource: ${error}`);
      console.error(error);
    },
  });

  const handleDelete = () => {
    if (!vente || !vente["@id"]) return;
    if (!window.confirm("Are you sure you want to delete this item?")) return;
    deleteMutation.mutate({ id: vente["@id"] });
  };

  return (
    <div className="container mx-auto px-4 max-w-2xl mt-4">
      <Link
        href="/ventes"
        className="text-sm text-cyan-500 font-bold hover:text-cyan-700"
      >
        {`< Back to list`}
      </Link>
      <h1 className="text-3xl my-2">
        {vente ? `Edit Vente ${vente["@id"]}` : `Create Vente`}
      </h1>
      <Formik
        initialValues={
          vente
            ? {
                ...vente,
              }
            : new Vente()
        }
        validate={() => {
          const errors = {};
          // add your validation logic here
          return errors;
        }}
        onSubmit={(values, { setSubmitting, setStatus, setErrors }) => {
          const isCreation = !values["@id"];
          saveMutation.mutate(
            { values },
            {
              onSuccess: () => {
                setStatus({
                  isValid: true,
                  msg: `Element ${isCreation ? "created" : "updated"}.`,
                });
                router.push("/ventes");
              },
              onError: (error) => {
                setStatus({
                  isValid: false,
                  msg: `${error.message}`,
                });
                if ("fields" in error) {
                  setErrors(error.fields);
                }
              },
              onSettled: () => {
                setSubmitting(false);
              },
            }
          );
        }}
      >
        {({
          values,
          status,
          errors,
          touched,
          handleChange,
          handleBlur,
          handleSubmit,
          isSubmitting,
        }) => (
          <form className="shadow-md p-4" onSubmit={handleSubmit}>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="vente_valeur"
              >
                valeur
              </label>
              <input
                name="valeur"
                id="vente_valeur"
                value={values.valeur ?? ""}
                type="number"
                step="0.1"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.valeur && touched.valeur ? "border-red-500" : ""
                }`}
                aria-invalid={
                  errors.valeur && touched.valeur ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="valeur"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="vente_date"
              >
                date
              </label>
              <input
                name="date"
                id="vente_date"
                value={values.date?.toLocaleString() ?? ""}
                type="dateTime"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.date && touched.date ? "border-red-500" : ""
                }`}
                aria-invalid={errors.date && touched.date ? "true" : undefined}
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="date"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="vente_departement"
              >
                departement
              </label>
              <input
                name="departement"
                id="vente_departement"
                value={values.departement ?? ""}
                type="text"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.departement && touched.departement
                    ? "border-red-500"
                    : ""
                }`}
                aria-invalid={
                  errors.departement && touched.departement ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="departement"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="vente_surface"
              >
                surface
              </label>
              <input
                name="surface"
                id="vente_surface"
                value={values.surface ?? ""}
                type="number"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.surface && touched.surface ? "border-red-500" : ""
                }`}
                aria-invalid={
                  errors.surface && touched.surface ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="surface"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="vente_codeType"
              >
                codeType
              </label>
              <input
                name="codeType"
                id="vente_codeType"
                value={values.codeType ?? ""}
                type="number"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.codeType && touched.codeType ? "border-red-500" : ""
                }`}
                aria-invalid={
                  errors.codeType && touched.codeType ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="codeType"
              />
            </div>
            <div className="mb-2">
              <label
                className="text-gray-700 block text-sm font-bold"
                htmlFor="vente_code_type"
              >
                code_type
              </label>
              <input
                name="code_type"
                id="vente_code_type"
                value={values.code_type ?? ""}
                type="number"
                placeholder=""
                className={`mt-1 block w-full ${
                  errors.code_type && touched.code_type ? "border-red-500" : ""
                }`}
                aria-invalid={
                  errors.code_type && touched.code_type ? "true" : undefined
                }
                onChange={handleChange}
                onBlur={handleBlur}
              />
              <ErrorMessage
                className="text-xs text-red-500 pt-1"
                component="div"
                name="code_type"
              />
            </div>
            {status && status.msg && (
              <div
                className={`border px-4 py-3 my-4 rounded ${
                  status.isValid
                    ? "text-cyan-700 border-cyan-500 bg-cyan-200/50"
                    : "text-red-700 border-red-400 bg-red-100"
                }`}
                role="alert"
              >
                {status.msg}
              </div>
            )}
            <button
              type="submit"
              className="inline-block mt-2 bg-cyan-500 hover:bg-cyan-700 text-sm text-white font-bold py-2 px-4 rounded"
              disabled={isSubmitting}
            >
              Submit
            </button>
          </form>
        )}
      </Formik>
      <div className="flex space-x-2 mt-4 justify-end">
        {vente && (
          <button
            className="inline-block mt-2 border-2 border-red-400 hover:border-red-700 hover:text-red-700 text-sm text-red-400 font-bold py-2 px-4 rounded"
            onClick={handleDelete}
          >
            Delete
          </button>
        )}
      </div>
    </div>
  );
};
