import {
  GetStaticPaths,
  GetStaticProps,
  NextComponentType,
  NextPageContext,
} from "next";
import DefaultErrorPage from "next/error";
import Head from "next/head";
import { useRouter } from "next/router";
import { dehydrate, QueryClient, useQuery } from "react-query";

import { Form } from "../../../components/nombreventes/Form";
import { PagedCollection } from "../../../types/collection";
import { NombreVentes } from "../../../types/NombreVentes";
import { fetch, FetchResponse, getPaths } from "../../../utils/dataAccess";

const getNombreVentes = async (id: string | string[] | undefined) =>
  id
    ? await fetch<NombreVentes>(`/ventes/nombre/${id}`)
    : Promise.resolve(undefined);

const Page: NextComponentType<NextPageContext> = () => {
  const router = useRouter();
  const { id } = router.query;

  const { data: { data: nombreventes } = {} } = useQuery<
    FetchResponse<NombreVentes> | undefined
  >(["nombreventes", id], () => getNombreVentes(id));

  if (!nombreventes) {
    return <DefaultErrorPage statusCode={404} />;
  }

  return (
    <div>
      <div>
        <Head>
          <title>
            {nombreventes && `Edit NombreVentes ${nombreventes["@id"]}`}
          </title>
        </Head>
      </div>
      <Form nombreventes={nombreventes} />
    </div>
  );
};

export const getStaticProps: GetStaticProps = async ({
  params: { id } = {},
}) => {
  if (!id) throw new Error("id not in query param");
  const queryClient = new QueryClient();
  await queryClient.prefetchQuery(["nombreventes", id], () =>
    getNombreVentes(id)
  );

  return {
    props: {
      dehydratedState: dehydrate(queryClient),
    },
    revalidate: 1,
  };
};

export const getStaticPaths: GetStaticPaths = async () => {
  const response = await fetch<PagedCollection<NombreVentes>>("/ventes/nombre");
  const paths = await getPaths(
    response,
    "ventes/nombre",
    "/nombreventess/[id]/edit"
  );

  return {
    paths,
    fallback: true,
  };
};

export default Page;
