import {
  GetStaticPaths,
  GetStaticProps,
  NextComponentType,
  NextPageContext,
} from "next";
import DefaultErrorPage from "next/error";
import Head from "next/head";
import { useRouter } from "next/router";
import { dehydrate, QueryClient, useQuery } from "react-query";

import { Form } from "../../../components/regionventes/Form";
import { PagedCollection } from "../../../types/collection";
import { RegionVentes } from "../../../types/RegionVentes";
import { fetch, FetchResponse, getPaths } from "../../../utils/dataAccess";

const getRegionVentes = async (id: string | string[] | undefined) =>
  id
    ? await fetch<RegionVentes>(`/ventes/regions/${id}`)
    : Promise.resolve(undefined);

const Page: NextComponentType<NextPageContext> = () => {
  const router = useRouter();
  const { id } = router.query;

  const { data: { data: regionventes } = {} } = useQuery<
    FetchResponse<RegionVentes> | undefined
  >(["regionventes", id], () => getRegionVentes(id));

  if (!regionventes) {
    return <DefaultErrorPage statusCode={404} />;
  }

  return (
    <div>
      <div>
        <Head>
          <title>
            {regionventes && `Edit RegionVentes ${regionventes["@id"]}`}
          </title>
        </Head>
      </div>
      <Form regionventes={regionventes} />
    </div>
  );
};

export const getStaticProps: GetStaticProps = async ({
  params: { id } = {},
}) => {
  if (!id) throw new Error("id not in query param");
  const queryClient = new QueryClient();
  await queryClient.prefetchQuery(["regionventes", id], () =>
    getRegionVentes(id)
  );

  return {
    props: {
      dehydratedState: dehydrate(queryClient),
    },
    revalidate: 1,
  };
};

export const getStaticPaths: GetStaticPaths = async () => {
  const response = await fetch<PagedCollection<RegionVentes>>(
    "/ventes/regions"
  );
  const paths = await getPaths(
    response,
    "ventes/regions",
    "/regionventess/[id]/edit"
  );

  return {
    paths,
    fallback: true,
  };
};

export default Page;
