import {
  GetStaticPaths,
  GetStaticProps,
  NextComponentType,
  NextPageContext,
} from "next";
import DefaultErrorPage from "next/error";
import Head from "next/head";
import { useRouter } from "next/router";
import { dehydrate, QueryClient, useQuery } from "react-query";

import { Show } from "../../../components/regionventes/Show";
import { PagedCollection } from "../../../types/collection";
import { RegionVentes } from "../../../types/RegionVentes";
import { fetch, FetchResponse, getPaths } from "../../../utils/dataAccess";
import { useMercure } from "../../../utils/mercure";

const getRegionVentes = async (id: string | string[] | undefined) =>
  id
    ? await fetch<RegionVentes>(`/ventes/regions/${id}`)
    : Promise.resolve(undefined);

const Page: NextComponentType<NextPageContext> = () => {
  const router = useRouter();
  const { id } = router.query;

  const {
    data: { data: regionventes, hubURL, text } = { hubURL: null, text: "" },
  } = useQuery<FetchResponse<RegionVentes> | undefined>(
    ["regionventes", id],
    () => getRegionVentes(id)
  );
  const regionventesData = useMercure(regionventes, hubURL);

  if (!regionventesData) {
    return <DefaultErrorPage statusCode={404} />;
  }

  return (
    <div>
      <div>
        <Head>
          <title>{`Show RegionVentes ${regionventesData["@id"]}`}</title>
        </Head>
      </div>
      <Show regionventes={regionventesData} text={text} />
    </div>
  );
};

export const getStaticProps: GetStaticProps = async ({
  params: { id } = {},
}) => {
  if (!id) throw new Error("id not in query param");
  const queryClient = new QueryClient();
  await queryClient.prefetchQuery(["regionventes", id], () =>
    getRegionVentes(id)
  );

  return {
    props: {
      dehydratedState: dehydrate(queryClient),
    },
    revalidate: 1,
  };
};

export const getStaticPaths: GetStaticPaths = async () => {
  const response = await fetch<PagedCollection<RegionVentes>>(
    "/ventes/regions"
  );
  const paths = await getPaths(
    response,
    "ventes/regions",
    "/regionventess/[id]"
  );

  return {
    paths,
    fallback: true,
  };
};

export default Page;
