import {
  GetStaticPaths,
  GetStaticProps,
  NextComponentType,
  NextPageContext,
} from "next";
import DefaultErrorPage from "next/error";
import Head from "next/head";
import { useRouter } from "next/router";
import { dehydrate, QueryClient, useQuery } from "react-query";

import { Form } from "../../../components/vente/Form";
import { PagedCollection } from "../../../types/collection";
import { Vente } from "../../../types/Vente";
import { fetch, FetchResponse, getPaths } from "../../../utils/dataAccess";

const getVente = async (id: string | string[] | undefined) =>
  id ? await fetch<Vente>(`/ventes/${id}`) : Promise.resolve(undefined);

const Page: NextComponentType<NextPageContext> = () => {
  const router = useRouter();
  const { id } = router.query;

  const { data: { data: vente } = {} } = useQuery<
    FetchResponse<Vente> | undefined
  >(["vente", id], () => getVente(id));

  if (!vente) {
    return <DefaultErrorPage statusCode={404} />;
  }

  return (
    <div>
      <div>
        <Head>
          <title>{vente && `Edit Vente ${vente["@id"]}`}</title>
        </Head>
      </div>
      <Form vente={vente} />
    </div>
  );
};

export const getStaticProps: GetStaticProps = async ({
  params: { id } = {},
}) => {
  if (!id) throw new Error("id not in query param");
  const queryClient = new QueryClient();
  await queryClient.prefetchQuery(["vente", id], () => getVente(id));

  return {
    props: {
      dehydratedState: dehydrate(queryClient),
    },
    revalidate: 1,
  };
};

export const getStaticPaths: GetStaticPaths = async () => {
  const response = await fetch<PagedCollection<Vente>>("/ventes");
  const paths = await getPaths(response, "ventes", "/ventes/[id]/edit");

  return {
    paths,
    fallback: true,
  };
};

export default Page;
