import {
  GetStaticPaths,
  GetStaticProps,
  NextComponentType,
  NextPageContext,
} from "next";
import DefaultErrorPage from "next/error";
import Head from "next/head";
import { useRouter } from "next/router";
import { dehydrate, QueryClient, useQuery } from "react-query";

import { Show } from "../../../components/vente/Show";
import { PagedCollection } from "../../../types/collection";
import { Vente } from "../../../types/Vente";
import { fetch, FetchResponse, getPaths } from "../../../utils/dataAccess";
import { useMercure } from "../../../utils/mercure";

const getVente = async (id: string | string[] | undefined) =>
  id ? await fetch<Vente>(`/ventes/${id}`) : Promise.resolve(undefined);

const Page: NextComponentType<NextPageContext> = () => {
  const router = useRouter();
  const { id } = router.query;

  const { data: { data: vente, hubURL, text } = { hubURL: null, text: "" } } =
    useQuery<FetchResponse<Vente> | undefined>(["vente", id], () =>
      getVente(id)
    );
  const venteData = useMercure(vente, hubURL);

  if (!venteData) {
    return <DefaultErrorPage statusCode={404} />;
  }

  return (
    <div>
      <div>
        <Head>
          <title>{`Show Vente ${venteData["@id"]}`}</title>
        </Head>
      </div>
      <Show vente={venteData} text={text} />
    </div>
  );
};

export const getStaticProps: GetStaticProps = async ({
  params: { id } = {},
}) => {
  if (!id) throw new Error("id not in query param");
  const queryClient = new QueryClient();
  await queryClient.prefetchQuery(["vente", id], () => getVente(id));

  return {
    props: {
      dehydratedState: dehydrate(queryClient),
    },
    revalidate: 1,
  };
};

export const getStaticPaths: GetStaticPaths = async () => {
  const response = await fetch<PagedCollection<Vente>>("/ventes");
  const paths = await getPaths(response, "ventes", "/ventes/[id]");

  return {
    paths,
    fallback: true,
  };
};

export default Page;
